﻿using Microsoft.Kinect;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace JointServer
{
    public partial class MainWindow : Window
    {
        private KinectSensor kinectSensor = null;
        private Body[] bodies = null;
        private BodyFrameReader bodyFrameReader = null;
        private KinectBodyView kinectBodyView = null;

        private Socket socket;
        private IPAddress serverAddr;
        private IPEndPoint endPoint;

        public MainWindow()
        {
            //Create UDP socket
            this.socket = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);
            this.serverAddr = IPAddress.Parse("127.0.0.1");
            this.endPoint = new IPEndPoint(this.serverAddr, 11000);

            //Open up kinect
            this.kinectSensor = KinectSensor.GetDefault();
            this.kinectSensor.Open();
            this.bodyFrameReader = this.kinectSensor.BodyFrameSource.OpenReader();
            this.bodyFrameReader.FrameArrived += this.Reader_BodyFrameArrived;

            this.kinectBodyView = new KinectBodyView(this.kinectSensor);

            this.InitializeComponent();

            this.DataContext = this;
            this.kinectBodyViewbox.DataContext = this.kinectBodyView;

        }

        private void MainWindow_Closing(object sender, CancelEventArgs e)
        {
            if (this.bodyFrameReader != null)
            {
                // BodyFrameReader is IDisposable
                this.bodyFrameReader.FrameArrived -= this.Reader_BodyFrameArrived;
                this.bodyFrameReader.Dispose();
                this.bodyFrameReader = null;
            }

            if (this.kinectSensor != null)
            {
                this.kinectSensor.Close();
                this.kinectSensor = null;
            }
        }

        private void Reader_BodyFrameArrived(object sender, BodyFrameArrivedEventArgs e)
        {
            bool dataReceived = false;

            using (BodyFrame bodyFrame = e.FrameReference.AcquireFrame())
            {
                if (bodyFrame != null)
                {
                    if (this.bodies == null)
                    {
                        this.bodies = new Body[bodyFrame.BodyCount];
                    }

                    bodyFrame.GetAndRefreshBodyData(this.bodies);
                    dataReceived = true;
                }
            }

            if (dataReceived)
            {
                // visualize the new body data
                this.kinectBodyView.UpdateBodyFrame(this.bodies);

                string dataToSend = "";
                for (int playerN = 0; playerN < this.bodies.Length; playerN++)
                {
                    if (!this.bodies[playerN].IsTracked) continue;

                    dataToSend += playerN;
                    foreach (JointType type in this.bodies[playerN].Joints.Keys)
                    {
                        Joint joint = this.bodies[playerN].Joints[type];
                        dataToSend += "#" + type + ";" + joint.TrackingState + ";" + joint.Position.X + ";" + joint.Position.Y + ";" + joint.Position.Z;

                    }

                    dataToSend += "&";

                }

                //data format 0#head;tracked;xx.x;yy.y;zz.z&0#head;tracked;xx.x;yy.y;zz.z&.......
                byte[] buffer = Encoding.ASCII.GetBytes(dataToSend);
                socket.SendTo(buffer, endPoint);



            }

        }

    }

}
