﻿using UnityEngine;
using System.Collections;
using System.Threading;
using System.Net.Sockets;
using System.Net;
using System.Text;
using System.Collections.Generic;
using System;

public class KinectSkeleton : MonoBehaviour {
    //==============================================================================
    private Thread UDPThread;
    private UdpClient UDPClient;
    private string UDPIP = "127.0.0.1";
    private int UDPPort = 11000;

    public Dictionary<int, PlayerSkeleton> players;

    //==============================================================================
    void Start()
    {
        Application.runInBackground = true;
        players = new Dictionary<int, PlayerSkeleton>();
        for (int i = 0; i < 6; i++) { players.Add(i, new PlayerSkeleton(i)); }
        InitializeUDPClient();

    }

    //==============================================================================
    private void InitializeUDPClient()
    {
        //Create thread for listening for UDP packets
        UDPThread = new Thread(new ThreadStart(ReceiveData));
        UDPThread.IsBackground = true;
        UDPThread.Start();

    }

    //==============================================================================
    private void ReceiveData()
    {
        //Create UDP Client
        UDPClient = new UdpClient(UDPPort);
        IPEndPoint endPoint = new IPEndPoint(IPAddress.Parse(UDPIP), UDPPort);

        //Forever
        while (true)
        {
            if (UDPClient.Client == null) { UDPThread.Abort(); break; }

            try
            {
                //Read next packet
                if (UDPClient.Available == 0) continue;
                byte[] data = UDPClient.Receive(ref endPoint);
                string packet = Encoding.UTF8.GetString(data);

                //0#head;tracked;xx.x;yy.y;zz.z&0#head;tracked;xx.x;yy.y;zz.z&.......
                string[] playerData = packet.Split(new char[] { '&' }, StringSplitOptions.RemoveEmptyEntries);

                //Debug.Log(packet);
                //Debug.Log("Tracking " + playerData.Length + " player(s)");

                //Reset status of all players
                foreach (PlayerSkeleton player in players.Values) player.isTracked = false;

                for (int i = 0; i < playerData.Length; i++)
                {
                    //0#head;tracked;xx.x;yy.y;zz.z
                    string[] jointData = playerData[i].Split(new char[] { '#' }, StringSplitOptions.RemoveEmptyEntries);
                    int id = Convert.ToInt32(jointData[0]);
                    
                    //Get player and set tracked to true
                    PlayerSkeleton player = players[id];
                    player.isTracked = true;

                    for (int j = 1; j < jointData.Length; j++)
                    {
                        //head;tracked;xx.x;yy.y;zz.z
                        string[] joint = jointData[j].Split(new char[] { ';' }, StringSplitOptions.RemoveEmptyEntries);
                        
                        //Extract JointType, status, and position
                        JointType type = (JointType) Enum.Parse(typeof(JointType), joint[0]);
                        string status = joint[1];
                        float x = float.Parse(joint[2], System.Globalization.CultureInfo.InvariantCulture);
                        float y = float.Parse(joint[3], System.Globalization.CultureInfo.InvariantCulture);
                        float z = float.Parse(joint[4], System.Globalization.CultureInfo.InvariantCulture);

                        //Update matching player joint status and position
                        if (status.Equals("Tracked")) player.joints[type].isTracked = true; else player.joints[type].isTracked = false;
                        player.joints[type].position.Set(x, y, z);

                    }

                }

            } catch (Exception e) { e.GetType(); }

        }

    }

    //==============================================================================
    void Update()
    {
        //Debug.Log("Player 0 - " + players[0].isTracked + "; Player 1 - " + players[1].isTracked + "; Player 2 - " + players[2].isTracked + "; Player 3 - " + players[3].isTracked + "; Player 4 - " + players[4].isTracked + "; Player 5 - " + players[5].isTracked);
    }

    //==============================================================================
    void OnDisable() { UDPClient.Close(); }

}


//===================================================================================================================================
//===================================================================================================================================
public enum JointType { SpineBase, SpineMid, Neck, Head, ShoulderLeft, ElbowLeft, WristLeft, HandLeft, ShoulderRight, ElbowRight, WristRight, HandRight, HipLeft, KneeLeft, AnkleLeft, FootLeft, HipRight, KneeRight, AnkleRight, FootRight, SpineShoulder, HandTipLeft, ThumbLeft, HandTipRight, ThumbRight, Count }
public class PlayerSkeleton
{
    public bool isTracked;
    public int id;
    public Dictionary<JointType, SkeletonJoint> joints = new Dictionary<JointType, SkeletonJoint>();
    public PlayerSkeleton(int id)
    {
        this.id = id;
        this.isTracked = false;
        foreach (JointType type in Enum.GetValues(typeof(JointType)))
        {
            if (type == JointType.Count) continue;
            joints.Add(type, new SkeletonJoint(type));
        }

    }

}

//===================================================================================================================================
//===================================================================================================================================
public class SkeletonJoint
{
    //Inferred, Tracked, NotTracked
    public bool isTracked;
    public Vector3 position;
    public JointType type;

    public SkeletonJoint(JointType type)
    {
        this.type = type;
        this.position = new Vector3(0, 0, 0);
        this.isTracked = false;

    }

}

